@@ -489,9 +489,28 @@ void add_chosen_column(WorkerMemory &workspace, InputData &input_data, ModelPara
             {
                 if (!workspace.weights_arr.size() && !workspace.weights_map.size())
                 {
-                    calc_mean_and_sd(workspace.ix_arr.data(), workspace.st, workspace.end,
-                                     input_data.numeric_data + workspace.col_chosen * input_data.nrows,
-                                     model_params.missing_action, workspace.ext_sd, workspace.ext_mean[workspace.ntaken]);
+                    if (model_params.missing_action == Fail && !model_params.standardize_data)
+                    {
+                        workspace.ext_mean[workspace.ntaken] = 0;
+                        workspace.ext_sd = 1;
+                    }
+
+                    else if (!model_params.standardize_data)
+                    {
+                        workspace.ext_sd = 1;
+                        workspace.ext_mean[workspace.ntaken]
+                            =
+                        calc_mean_only(workspace.ix_arr.data(), workspace.st, workspace.end,
+                                     input_data.numeric_data + workspace.col_chosen * input_data.nrows);
+                    }
+
+                    else
+                    {
+                        calc_mean_and_sd(workspace.ix_arr.data(), workspace.st, workspace.end,
+                                         input_data.numeric_data + workspace.col_chosen * input_data.nrows,
+                                         model_params.missing_action, workspace.ext_sd, workspace.ext_mean[workspace.ntaken]);
+                    }
+
                     add_linear_comb(workspace.ix_arr.data(), workspace.st, workspace.end, workspace.comb_val.data(),
                                     input_data.numeric_data + workspace.col_chosen * input_data.nrows,
                                     workspace.ext_coef[workspace.ntaken], workspace.ext_sd, workspace.ext_mean[workspace.ntaken],
@@ -500,11 +519,31 @@ void add_chosen_column(WorkerMemory &workspace, InputData &input_data, ModelPara
                 }
                 else if (workspace.weights_arr.size())
                 {
-                    calc_mean_and_sd_weighted(workspace.ix_arr.data(), workspace.st, workspace.end,
-                                              input_data.numeric_data + workspace.col_chosen * input_data.nrows,
-                                              workspace.weights_arr,
-                                              model_params.missing_action, workspace.ext_sd,
-                                              workspace.ext_mean[workspace.ntaken]);
+                    if (model_params.missing_action == Fail && !model_params.standardize_data)
+                    {
+                        workspace.ext_mean[workspace.ntaken] = 0;
+                        workspace.ext_sd = 1;
+                    }
+
+                    else if (!model_params.standardize_data)
+                    {
+                        workspace.ext_sd = 1;
+                        workspace.ext_mean[workspace.ntaken]
+                            =
+                        calc_mean_only_weighted(workspace.ix_arr.data(), workspace.st, workspace.end,
+                                                input_data.numeric_data + workspace.col_chosen * input_data.nrows,
+                                                workspace.weights_arr);
+                    }
+
+                    else
+                    {
+                        calc_mean_and_sd_weighted(workspace.ix_arr.data(), workspace.st, workspace.end,
+                                                  input_data.numeric_data + workspace.col_chosen * input_data.nrows,
+                                                  workspace.weights_arr,
+                                                  model_params.missing_action, workspace.ext_sd,
+                                                  workspace.ext_mean[workspace.ntaken]);
+                    }
+                    
                     add_linear_comb_weighted(workspace.ix_arr.data(), workspace.st, workspace.end, workspace.comb_val.data(),
                                              input_data.numeric_data + workspace.col_chosen * input_data.nrows,
                                              workspace.ext_coef[workspace.ntaken], workspace.ext_sd, workspace.ext_mean[workspace.ntaken],
@@ -515,11 +554,31 @@ void add_chosen_column(WorkerMemory &workspace, InputData &input_data, ModelPara
 
                 else
                 {
-                    calc_mean_and_sd_weighted(workspace.ix_arr.data(), workspace.st, workspace.end,
-                                              input_data.numeric_data + workspace.col_chosen * input_data.nrows,
-                                              workspace.weights_map,
-                                              model_params.missing_action, workspace.ext_sd,
-                                              workspace.ext_mean[workspace.ntaken]);
+                    if (model_params.missing_action == Fail && !model_params.standardize_data)
+                    {
+                        workspace.ext_mean[workspace.ntaken] = 0;
+                        workspace.ext_sd = 1;
+                    }
+
+                    else if (!model_params.standardize_data)
+                    {
+                        workspace.ext_sd = 1;
+                        workspace.ext_mean[workspace.ntaken]
+                            =
+                        calc_mean_only_weighted(workspace.ix_arr.data(), workspace.st, workspace.end,
+                                                input_data.numeric_data + workspace.col_chosen * input_data.nrows,
+                                                workspace.weights_map);
+                    }
+
+                    else
+                    {
+                        calc_mean_and_sd_weighted(workspace.ix_arr.data(), workspace.st, workspace.end,
+                                                  input_data.numeric_data + workspace.col_chosen * input_data.nrows,
+                                                  workspace.weights_map,
+                                                  model_params.missing_action, workspace.ext_sd,
+                                                  workspace.ext_mean[workspace.ntaken]);
+                    }
+
                     add_linear_comb_weighted(workspace.ix_arr.data(), workspace.st, workspace.end, workspace.comb_val.data(),
                                              input_data.numeric_data + workspace.col_chosen * input_data.nrows,
                                              workspace.ext_coef[workspace.ntaken], workspace.ext_sd, workspace.ext_mean[workspace.ntaken],
@@ -533,9 +592,28 @@ void add_chosen_column(WorkerMemory &workspace, InputData &input_data, ModelPara
             {
                 if (!workspace.weights_arr.size() && !workspace.weights_map.size())
                 {
-                    calc_mean_and_sd(workspace.ix_arr.data(), workspace.st, workspace.end, workspace.col_chosen,
-                                     input_data.Xc, input_data.Xc_ind, input_data.Xc_indptr,
-                                     workspace.ext_sd, workspace.ext_mean[workspace.ntaken]);
+                    if (model_params.missing_action == Fail && !model_params.standardize_data)
+                    {
+                        workspace.ext_mean[workspace.ntaken] = 0;
+                        workspace.ext_sd = 1;
+                    }
+
+                    else if (!model_params.standardize_data)
+                    {
+                        workspace.ext_sd = 1;
+                        workspace.ext_mean[workspace.ntaken]
+                            =
+                        calc_mean_only(workspace.ix_arr.data(), workspace.st, workspace.end, workspace.col_chosen,
+                                     input_data.Xc, input_data.Xc_ind, input_data.Xc_indptr);
+                    }
+
+                    else
+                    {
+                        calc_mean_and_sd(workspace.ix_arr.data(), workspace.st, workspace.end, workspace.col_chosen,
+                                         input_data.Xc, input_data.Xc_ind, input_data.Xc_indptr,
+                                         workspace.ext_sd, workspace.ext_mean[workspace.ntaken]);
+                    }
+
                     add_linear_comb(workspace.ix_arr.data(), workspace.st, workspace.end,
                                     workspace.col_chosen, workspace.comb_val.data(),
                                     input_data.Xc, input_data.Xc_ind, input_data.Xc_indptr,
@@ -546,10 +624,30 @@ void add_chosen_column(WorkerMemory &workspace, InputData &input_data, ModelPara
 
                 else if (workspace.weights_arr.size())
                 {
-                    calc_mean_and_sd_weighted(workspace.ix_arr.data(), workspace.st, workspace.end, workspace.col_chosen,
-                                              input_data.Xc, input_data.Xc_ind, input_data.Xc_indptr,
-                                              workspace.ext_sd, workspace.ext_mean[workspace.ntaken],
-                                              workspace.weights_arr);
+                    if (model_params.missing_action == Fail && !model_params.standardize_data)
+                    {
+                        workspace.ext_mean[workspace.ntaken] = 0;
+                        workspace.ext_sd = 1;
+                    }
+
+                    else if (!model_params.standardize_data)
+                    {
+                        workspace.ext_sd = 1;
+                        workspace.ext_mean[workspace.ntaken]
+                            =
+                        calc_mean_only_weighted(workspace.ix_arr.data(), workspace.st, workspace.end, workspace.col_chosen,
+                                                input_data.Xc, input_data.Xc_ind, input_data.Xc_indptr,
+                                                workspace.weights_arr);
+                    }
+
+                    else
+                    {
+                        calc_mean_and_sd_weighted(workspace.ix_arr.data(), workspace.st, workspace.end, workspace.col_chosen,
+                                                  input_data.Xc, input_data.Xc_ind, input_data.Xc_indptr,
+                                                  workspace.ext_sd, workspace.ext_mean[workspace.ntaken],
+                                                  workspace.weights_arr);
+                    }
+
                     add_linear_comb_weighted(workspace.ix_arr.data(), workspace.st, workspace.end,
                                              workspace.col_chosen, workspace.comb_val.data(),
                                              input_data.Xc, input_data.Xc_ind, input_data.Xc_indptr,
@@ -561,10 +659,30 @@ void add_chosen_column(WorkerMemory &workspace, InputData &input_data, ModelPara
 
                 else
                 {
-                    calc_mean_and_sd_weighted(workspace.ix_arr.data(), workspace.st, workspace.end, workspace.col_chosen,
-                                              input_data.Xc, input_data.Xc_ind, input_data.Xc_indptr,
-                                              workspace.ext_sd, workspace.ext_mean[workspace.ntaken],
-                                              workspace.weights_map);
+                    if (model_params.missing_action == Fail && !model_params.standardize_data)
+                    {
+                        workspace.ext_mean[workspace.ntaken] = 0;
+                        workspace.ext_sd = 1;
+                    }
+
+                    else if (!model_params.standardize_data)
+                    {
+                        workspace.ext_sd = 1;
+                        workspace.ext_mean[workspace.ntaken]
+                            =
+                        calc_mean_only_weighted(workspace.ix_arr.data(), workspace.st, workspace.end, workspace.col_chosen,
+                                                input_data.Xc, input_data.Xc_ind, input_data.Xc_indptr,
+                                                workspace.weights_map);
+                    }
+
+                    else
+                    {
+                        calc_mean_and_sd_weighted(workspace.ix_arr.data(), workspace.st, workspace.end, workspace.col_chosen,
+                                                  input_data.Xc, input_data.Xc_ind, input_data.Xc_indptr,
+                                                  workspace.ext_sd, workspace.ext_mean[workspace.ntaken],
+                                                  workspace.weights_map);
+                    }
+                    
                     add_linear_comb_weighted(workspace.ix_arr.data(), workspace.st, workspace.end,
                                              workspace.col_chosen, workspace.comb_val.data(),
                                              input_data.Xc, input_data.Xc_ind, input_data.Xc_indptr,
