@@ -692,13 +692,15 @@ void predict_iforest(real_t numeric_data[], int categ_data[],
                      real_t Xr[], sparse_ix Xr_ind[], sparse_ix Xr_indptr[],
                      size_t nrows, int nthreads, bool standardize,
                      IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
-                     double output_depths[],   sparse_ix tree_num[]);
+                     double output_depths[],   sparse_ix tree_num[],
+                     double per_tree_depths[]);
 template <class PredictionData, class sparse_ix>
 void traverse_itree_no_recurse(std::vector<IsoTree>  &tree,
                                IsoForest             &model_outputs,
                                PredictionData        &prediction_data,
                                double                &output_depth,
                                sparse_ix *restrict   tree_num,
+                               double *restrict      tree_depth,
                                size_t                row);
 template <class PredictionData, class sparse_ix, class ImputedData>
 double traverse_itree(std::vector<IsoTree>     &tree,
@@ -709,6 +711,7 @@ double traverse_itree(std::vector<IsoTree>     &tree,
                       double                   curr_weight,
                       size_t                   row,
                       sparse_ix *restrict      tree_num,
+                      double *restrict         tree_depth,
                       size_t                   curr_lev);
 template <class PredictionData, class sparse_ix>
 void traverse_hplane_fast(std::vector<IsoHPlane>  &hplane,
@@ -716,6 +719,7 @@ void traverse_hplane_fast(std::vector<IsoHPlane>  &hplane,
                           PredictionData          &prediction_data,
                           double                  &output_depth,
                           sparse_ix *restrict     tree_num,
+                          double *restrict        tree_depth,
                           size_t                  row);
 template <class PredictionData, class sparse_ix, class ImputedData>
 void traverse_hplane(std::vector<IsoHPlane>   &hplane,
@@ -725,17 +729,20 @@ void traverse_hplane(std::vector<IsoHPlane>   &hplane,
                      std::vector<ImputeNode> *impute_nodes,
                      ImputedData             *imputed_data,
                      sparse_ix *restrict      tree_num,
+                     double *restrict         tree_depth,
                      size_t                   row);
 template <class real_t, class sparse_ix>
 void batched_csc_predict(PredictionData<real_t, sparse_ix> &prediction_data, int nthreads,
                          IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
-                         double output_depths[],   sparse_ix tree_num[]);
+                         double output_depths[],   sparse_ix tree_num[],
+                         double per_tree_depths[]);
 template <class PredictionData, class sparse_ix>
 void traverse_itree_csc(WorkerForPredictCSC   &workspace,
                         std::vector<IsoTree>  &trees,
                         IsoForest             &model_outputs,
                         PredictionData        &prediction_data,
                         sparse_ix             *tree_num,
+                        double                *per_tree_depths,
                         size_t                curr_tree,
                         bool                  has_range_penalty);
 template <class PredictionData, class sparse_ix>
@@ -744,6 +751,7 @@ void traverse_hplane_csc(WorkerForPredictCSC      &workspace,
                          ExtIsoForest             &model_outputs,
                          PredictionData           &prediction_data,
                          sparse_ix                *tree_num,
+                         double                   *per_tree_depths,
                          size_t                   curr_tree,
                          bool                     has_range_penalty);
 template <class PredictionData>
